#include <windows.h>
#include <tchar.h>
#include <memory.h>
#include <commctrl.h>
#include "resource.h"
#include "ppongo.h"


TCHAR				szAppName[] = TEXT("Pocket Pongo");
HWND				hCmdBar;

HWND				hMainWnd;
HWND				hTermWnd;
HWND				hRINGWnd;
HINSTANCE			hInstance;
HANDLE				hPort = INVALID_HANDLE_VALUE;	// The serial port handle.
BOOL				v_bStopThread;					// To tell the thread to go away.
TCHAR				v_szDevName[128];				// The name of the device
TERM_SCREEN			Screen;
HFONT				v_hFont;						// The font to use.
CRITICAL_SECTION	critSect;
LPWSTR				pswCommandLine;

#define		BUTTONBAR_HEIGHT	16
#define		BUTTON_HEIGHT		14

/************************************************************************
  TermWndProc -Main Windows Procedure
************************************************************************/

LRESULT CALLBACK TermWndProc ( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam )
{
	HDC			hDC;
	PAINTSTRUCT	ps;

	switch (message) 
	{
		case WM_DESTROY :
			PostQuitMessage(0);
			break;
		case WM_SETFOCUS :
			Screen.HaveFocus = TRUE;
			CreateCaret(hWnd, NULL, Screen.CharWidth, Screen.CharHeight);
			if (ON_SCREEN(Screen.CursorRow, Screen.CursorCol)) 
			{
				SetCaretPos(COLX(Screen.CursorCol), ROWY(Screen.CursorRow));
				ShowCaret(hWnd);		
			}
			break;
		case WM_KILLFOCUS :
			Screen.HaveFocus = FALSE;
			HideCaret(hWnd);
			DestroyCaret();
			break;
		case WM_SIZE :
			// Resize everthing.
			InitScreenSettings (hWnd);
			break;
		case WM_PAINT :
			EnterCriticalSection(&critSect);
			HideCaret(hTermWnd);
			hDC = BeginPaint(hWnd, &ps);
			PaintScreen(hWnd, hDC, &(ps.rcPaint));
			EndPaint(hWnd, &ps);
			ShowCaret(hTermWnd);
			LeaveCriticalSection(&critSect);
			break;
		case WM_CHAR :
			if (INVALID_HANDLE_VALUE != hPort) 
			{
				PortWrite ((BYTE)(wParam&0xFF));
			} 
			else 
			{
// TEST CODE
				ProcessChar ((BYTE)(wParam&0xFF));
// END TEST CODE
			}
			break;

		case WM_VSCROLL :
			switch (LOWORD(wParam)) 
			{
				case SB_LINEUP:
					ScrollDown (-1);
					break;
				case SB_LINEDOWN:
					ScrollDown (1);
					break;
				case SB_PAGEUP:
					ScrollDown (-(Screen.ScreenRows-1));
					break;
				case SB_PAGEDOWN:
					ScrollDown (Screen.ScreenRows-1);
					break;
				case SB_TOP:
					ScrollDown (-Screen.NumRows);
					break;
				case SB_BOTTOM:
					ScrollDown (Screen.NumRows);
					break;
				case SB_THUMBPOSITION :
					ScrollDown (HIWORD(wParam)-Screen.TopRow);
					break;
				case SB_THUMBTRACK:
					// Let's not handle this
					ScrollDown(HIWORD(wParam)-Screen.TopRow);
					break;
				case SB_ENDSCROLL:
					break;
			}
			break;
		case WM_HSCROLL :
			switch (LOWORD(wParam)) 
			{
				case SB_LINELEFT :
					ScrollRight (-1);
					break;
				case SB_LINERIGHT :
					ScrollRight (1);
					break;
				case SB_PAGELEFT :
					ScrollRight (-(Screen.ScreenCols-1));
					break;
				case SB_PAGERIGHT :
					ScrollRight (Screen.ScreenCols-1);
					break;
				case SB_LEFT :
					ScrollRight (-Screen.NumCols);
					break;
				case SB_RIGHT :
					ScrollRight (Screen.NumCols);
					break;
				case SB_THUMBPOSITION :
					ScrollRight (HIWORD(wParam)-Screen.LeftCol);
					break;
				case SB_THUMBTRACK :
					// Let's not handle this
					break;
				case SB_ENDSCROLL :
					break;
			}
			break;

		default:
			return (DefWindowProc(hWnd, message, wParam, lParam));
	}
	return (0);
}


/*********************************************************************
      MainWndProc() - Main windows procedure
**********************************************************************/

LRESULT CALLBACK MainWndProc ( HWND hWnd, UINT message, WPARAM wParam, LPARAM lParam )
{
	HDC			hDC;
	PAINTSTRUCT	ps;
	RECT		rc;
	HBRUSH		hBrush;

	switch (message) 
	{
		case WM_CREATE :
			break;
		case WM_DESTROY :
			v_bStopThread = TRUE;
			SetCommMask (hPort, 0);
			PostQuitMessage(0);
			break;
		case WM_PAINT :
			hDC = BeginPaint(hWnd, &ps);
			GetClientRect (hWnd, &rc);
			rc.bottom = BUTTONBAR_HEIGHT;
			hBrush = (HBRUSH)GetStockObject (LTGRAY_BRUSH);
			FillRect (hDC, &rc, hBrush);
			DeleteObject (hBrush);
			rc.top = BUTTONBAR_HEIGHT - 1;
			hBrush = (HBRUSH)GetStockObject (BLACK_BRUSH);
			FillRect (hDC, &rc, hBrush);
			DeleteObject (hBrush);
			EndPaint(hWnd, &ps);

			break;
		case WM_SETFOCUS :
			// Give focus to the terminal window
			SetFocus (hTermWnd);
			break;
		case WM_SIZE :
			GetClientRect (hWnd, &rc);
			rc.top = CommandBar_Height(hCmdBar);
			rc.top += BUTTONBAR_HEIGHT;
			SetWindowPos (hTermWnd, HWND_TOP, rc.left, rc.top, rc.right-rc.left, rc.bottom-rc.top,0);
			ShowWindow (hTermWnd, SW_SHOWNORMAL);
			break;
		case WM_COMMAND :
			switch (LOWORD(wParam)) 
			{
				case ID_FILE_OPEN_COMM:
					InitPort();
					break;
				case ID_FILE_CLOSE_COMM:
					ClosePort();
					break;
				case ID_FILE_EXIT :
					DestroyWindow(hWnd);
					break;
				case ID_EDIT_CLEARBUFFER :
					ClearScreen();
					break;
				case ID_SETTINGS :
					DialogBox (hInstance, MAKEINTRESOURCE(IDD_COMMUNICATIONS), hWnd, CommDlgProc);
					break;
				case ID_HELP_ABOUTTTY :
					MessageBox (hWnd, TEXT("Pocket Pongo 1.1 for Windows CE\r\nSimple Packet Radio Terminal Program\r\n(c) 1999~2000 Marco Savegnago IW3FQG\r\ne-mail: iw3fqg@amsat.org"), TEXT("About Pocket Pongo"), MB_OK);
					break;
				default :
					MessageBox (hWnd, TEXT("Unhandled WM_COMMAND message\r\n"), TEXT("ERROR"), MB_OK);
					break;
			}
			break;
		default:
			return (DefWindowProc(hWnd, message, wParam, lParam));
	}
	return (0);
}

/*****************************************************************************
             InitApplication()
******************************************************************************/
BOOL InitApplication(HINSTANCE hInst) 
{
    WNDCLASS  wc;
	
	InitializeCriticalSection(&critSect);

	InitCommonControls();

    wc.style = CS_HREDRAW | CS_VREDRAW;
    wc.lpfnWndProc = (WNDPROC) MainWndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInst;
    wc.hIcon = (HICON) LoadIcon(hInstance,MAKEINTRESOURCE(IDI_TERMINAL));;
    wc.hCursor = NULL;
    wc.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName = 0;
    wc.lpszClassName = szAppName;
	
	if (!RegisterClass(&wc)) 
	{
		return FALSE;
	}
    
	wc.style = CS_VREDRAW|CS_HREDRAW;
    wc.lpfnWndProc = (WNDPROC) TermWndProc;
    wc.cbClsExtra = 0;
    wc.cbWndExtra = 0;
    wc.hInstance = hInst;
    wc.hIcon = (HICON) NULL;
    wc.hCursor = NULL;
    wc.hbrBackground = (HBRUSH) GetStockObject(WHITE_BRUSH);
    wc.lpszMenuName = 0;
    wc.lpszClassName = TEXT("TermClass");
	
	return RegisterClass(&wc);
}

/***************************************************************************
     InitInstance()
***************************************************************************/

BOOL InitInstance(HINSTANCE hInst, int nCmdShow)
{
	DWORD	ErrorCode;
	RECT	rc;

    hMainWnd = CreateWindowEx (0, 
								szAppName, 
								TEXT("POCKET PONGO"), 
								WS_CLIPCHILDREN,
								CW_USEDEFAULT, 
								0, 
								CW_USEDEFAULT, 
								0,
								NULL, 
								NULL,
								hInst, 
								NULL
								);

	
	if (NULL == hMainWnd) 
	{
		ErrorCode = GetLastError();
		return FALSE;
	}
	
	GetClientRect (hMainWnd, &rc);

	hCmdBar = CommandBar_Create (hInst, hMainWnd, 1);
	CommandBar_InsertMenubar(hCmdBar, hInst, IDR_MAIN_MENU, 0);
	CommandBar_AddAdornments(hCmdBar, 0, ID_FILE_EXIT);
	rc.top = CommandBar_Height(hCmdBar);

	ShowWindow(hMainWnd, nCmdShow);
	UpdateWindow (hMainWnd);

	// Create the terminal window.
	hTermWnd = CreateWindowEx (0, TEXT("TermClass"),
							   TEXT("POCKET TERMINAL"),
							   WS_CHILD | WS_VSCROLL | WS_HSCROLL | WS_VISIBLE | WS_BORDER,
							   rc.left, rc.top, rc.right - rc.left,
							   rc.bottom-rc.top,
							   hMainWnd, NULL,
							   hInst, NULL);
	if (NULL == hTermWnd) 
	{
		ErrorCode = GetLastError();
		return FALSE;
	}

	SetFocus (hTermWnd);

	return TRUE;
	
}

/********************************************************************
            WinMain()
*********************************************************************/

int WINAPI WinMain(HINSTANCE hInst, HINSTANCE hInstPrev, LPWSTR pszCmdLine,	int nCmdShow)
{
    MSG msg;
	DWORD fdwCommModemStatus;

	hInstance = hInst;
	
	pswCommandLine	=	pszCmdLine;

	if (!pswCommandLine || TEXT('\0') == *pswCommandLine) 
		_tcscpy (v_szDevName, TEXT("COM1:"));
	else
		_tcscpy (v_szDevName, pswCommandLine);

	if(!hInstPrev) 
	{
		if(!InitApplication(hInst))
			return FALSE;
	}

	if (!InitInstance (hInst, nCmdShow)) 
	{
		if (hTermWnd) 
		{
			DestroyWindow(hTermWnd);
		}
		
		goto ExitMain;
	}

	// Initialize the screen
	if (TRUE != InitScreenSettings(hTermWnd)) 
	{
		MessageBox (NULL, TEXT("Unable to allocate screen buffer\r\n"), TEXT("Fatal Error"), MB_OK);
		DestroyWindow(hTermWnd);
		goto ExitMain;
	}

	// Clear the screen.
	ClearScreen();

	// Init the Indicator lights.
	GetCommModemStatus (hPort, &fdwCommModemStatus);
	
    while ( GetMessage(&msg, NULL, 0, 0) == TRUE ) 
	{
        TranslateMessage(&msg);
        DispatchMessage(&msg);
    }

ExitMain:

	//	Free the critical section
	DeleteCriticalSection(&critSect);
	
	if (v_hFont) 
	{
		// Free the font.
		DeleteObject ((HGDIOBJ)v_hFont);
	}

	return TRUE;
}
