#include <windows.h>
#include <tchar.h>
#include <memory.h>

#include "resource.h"
#include "ppongo.h"

void ClearScreen()
{
	int	i;

	for (i=0; i < Screen.NumRows*(Screen.NumCols+1); i++) 
	{
		Screen.ScreenBuff[i] = TEXT(' ');
	}

	for (i=0; i < Screen.NumRows; i++) 
	{
		Screen.ScreenBuff[i*(Screen.NumCols+1) + Screen.NumCols] = TEXT(' ');
	}

	Screen.TopRow = 0;
	Screen.LeftCol = 0;
	Screen.CursorRow = 0;
	Screen.CursorCol = 0;
	
	InvalidateRect (hTermWnd, NULL, TRUE);
	UpdateWindow (hTermWnd);
}

void FillScreen()
{
	int	i;
	
//	memset ((char *)Screen.ScreenBuff, TEXT(' '), Screen.NumRows*(Screen.NumCols+1)*sizeof(TCHAR));

	for (i=0; i < Screen.NumRows*(Screen.NumCols+1); i++) 
	{
		Screen.ScreenBuff[i] = TEXT(' ');
	}
	
	for (i=0; i < Screen.NumRows; i++) 
	{
		memcpy (&(SCREENPOS(i,0)), TEXT("Test row 00"), 11*sizeof(TCHAR));
		Screen.ScreenBuff[i*(Screen.NumCols+1)+9] += (i / 10) % 10;
		Screen.ScreenBuff[i*(Screen.NumCols+1)+10] += i % 10;
		
		Screen.ScreenBuff[i*(Screen.NumCols+1) + Screen.NumCols] = TEXT(' ');
	}

	Screen.TopRow		= 0;
	Screen.LeftCol		= 0;
	Screen.CursorRow	= 0;
	Screen.CursorCol	= 0;
	
	InvalidateRect (hTermWnd, NULL, TRUE);
	UpdateWindow (hTermWnd);
}


void PaintScreen (HWND hWnd, HDC	hDC, RECT *pRect)
{
	int		Row;
	DWORD	X, Y;
	HFONT	oldhfont = NULL;
	DWORD	StartCol, EndCol;
	
	oldhfont = (HFONT)SelectObject(hDC, v_hFont);
	(void)SetTextColor(hDC,RGB(0,0,0));
	(void)SetBkColor(hDC,RGB(0xff, 0xff, 0xff));

	// Which column to start
	StartCol = (pRect->left + Screen.CharWidth*Screen.LeftCol) / Screen.CharWidth;
	EndCol = (pRect->right + (Screen.CharWidth+1)*Screen.LeftCol) / Screen.CharWidth;
	
	if (EndCol > Screen.NumCols) 
	{
		EndCol = Screen.NumCols;
	}
	
	X = (StartCol-Screen.LeftCol)*Screen.CharWidth;

	for (Row=Screen.TopRow, Y = 0; Row < (Screen.TopRow + Screen.ScreenRows); Row++, Y += Screen.CharHeight) 
	{
		// Should we draw this row?
		if ((ROWY(Row+1) >= pRect->top) && (ROWY(Row) <= pRect->bottom)) 
		{
			ExtTextOut(hDC, X, Y, 0, 0, &(SCREENPOS(Row,StartCol)), EndCol - StartCol, NULL);
		}
	}

	if (oldhfont != NULL) 
	{
		SelectObject (hDC, oldhfont);
	}

	if (Screen.HaveFocus) 
	{
		if (ON_SCREEN(Screen.CursorRow, Screen.CursorCol)) 
		{
			SetCaretPos(COLX(Screen.CursorCol), ROWY(Screen.CursorRow));
			ShowCaret(hTermWnd);
		} 
		else 
		{
			HideCaret(hTermWnd);
		}
	}
	
}

BOOL InitScreenSettings (HWND hWnd)
{
	HDC				hDC = GetDC (hWnd);
	HFONT			oldhFont;
	TEXTMETRIC		TextMetrics;
	RECT			rc;
    LOGFONT         lf;
    SCROLLINFO      si;

	// First Time?
	if (NULL == Screen.ScreenBuff) 
	{
		Screen.NumRows = INIT_SCREEN_ROWS;
		Screen.NumCols = INIT_SCREEN_COLS;
		
		Screen.ScreenBuff = (PTCHAR)LocalAlloc (LPTR, Screen.NumRows*(Screen.NumCols+1)*sizeof(TCHAR));

		if (NULL == Screen.ScreenBuff) 
		{
			return FALSE;
		}
		
		memset ((char *)&lf, 0, sizeof(lf));
		
		lf.lfPitchAndFamily = FIXED_PITCH | FF_MODERN;
		lf.lfHeight = 13;
		v_hFont = CreateFontIndirect (&lf);
	}

	oldhFont = (HFONT)SelectObject (hDC, v_hFont);

	GetTextMetrics (hDC, &TextMetrics);

	Screen.CharHeight = (USHORT)TextMetrics.tmHeight;
	Screen.CharWidth = (USHORT)TextMetrics.tmMaxCharWidth;

	if (NULL != oldhFont)  
	{
		(void)SelectObject(hDC,(HGDIOBJ)oldhFont);
	}

	ReleaseDC (hWnd, hDC);

	Screen.TopRow = 0;
	Screen.LeftCol = 0;
	Screen.CursorRow = 0;
	Screen.CursorCol = 0;

	GetClientRect (hWnd, &rc);

	Screen.ScreenRows = rc.bottom / Screen.CharHeight;
	Screen.ScreenCols = rc.right / Screen.CharWidth;
	
	// Set up the scroll bars.
	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_RANGE|SIF_POS|SIF_PAGE;
	si.nMin = 0;
	si.nMax = Screen.NumRows;
	si.nPage = Screen.ScreenRows;
	si.nPos = 0;
	SetScrollInfo(hWnd, SB_VERT, &si, TRUE);

	si.fMask = SIF_RANGE|SIF_POS|SIF_PAGE;
	si.nMin = 0;
	si.nMax = Screen.NumCols;
	si.nPage = Screen.ScreenCols;
	si.nPos = 0;
	SetScrollInfo(hWnd, SB_HORZ, &si, TRUE);
	
	return TRUE;
}

void ScrollDown(int Rows)
{
	DWORD   	OldTop = Screen.TopRow;
	SCROLLINFO  si;
	RECT	    rc;

	// Check some limit conditions.
	if ((Rows < 0) && (-Rows > Screen.TopRow)) 
	{
		Screen.TopRow = 0;
	} 
	else 
	{
		// Move the top row.
		Screen.TopRow += Rows;
		
		if (Screen.TopRow > (Screen.NumRows - Screen.ScreenRows)) 
		{
			Screen.TopRow = Screen.NumRows - Screen.ScreenRows;
		}
	}

	// Is there anything to do?
	if (OldTop == Screen.TopRow) 
	{
		return;
	}

	GetClientRect(hTermWnd, &rc);
	HideCaret(hTermWnd);
	
	ScrollWindowEx(hTermWnd, 0, (OldTop - Screen.TopRow)*Screen.CharHeight, &rc, &rc, NULL, NULL, SW_INVALIDATE|SW_ERASE);
	
	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_POS;
	si.nPos = Screen.TopRow;
	SetScrollInfo(hTermWnd, SB_VERT, &si, TRUE);
	ShowCaret(hTermWnd);
}

void ScrollRight(int Cols)
{
	DWORD		OldLeft = Screen.LeftCol;
	SCROLLINFO  si;
	RECT	    rc;
	
	if ((Cols < 0) && (-Cols > Screen.LeftCol)) 
	{
		Screen.LeftCol = 0;
	} 
	else 
	{
		Screen.LeftCol += Cols;
	
		if (Screen.LeftCol > (Screen.NumCols - Screen.ScreenCols)) 
		{
			Screen.LeftCol = Screen.NumCols - Screen.ScreenCols;
		}
	}

	if (OldLeft == Screen.LeftCol) 
	{
		return;
	}

	GetClientRect(hTermWnd, &rc);
	HideCaret(hTermWnd);
	
	ScrollWindowEx(hTermWnd, (OldLeft - Screen.LeftCol)*Screen.CharWidth, 0, &rc, &rc, NULL, NULL, SW_INVALIDATE|SW_ERASE);

	si.cbSize = sizeof(SCROLLINFO);
	si.fMask = SIF_POS;
	si.nPos = Screen.LeftCol;
	SetScrollInfo(hTermWnd, SB_HORZ, &si, TRUE);
	ShowCaret(hTermWnd);
}

void PutCursorOnScreen()
{
	if (ON_SCREEN(Screen.CursorRow, Screen.CursorCol)) 
	{
		return;
	}
	
	if (Screen.CursorRow < Screen.TopRow) 
	{
		ScrollDown (Screen.CursorRow - Screen.TopRow);
	} 
	else if (Screen.CursorRow > (Screen.TopRow+Screen.ScreenRows-1)) 
	{
		ScrollDown (Screen.CursorRow - (Screen.TopRow+Screen.ScreenRows-1));
	}
	
	if (Screen.CursorCol < Screen.LeftCol) 
	{
		ScrollRight (Screen.CursorCol - Screen.LeftCol);
	} 
	else if (Screen.CursorCol > (Screen.LeftCol+Screen.ScreenCols-1)) 
	{
		ScrollRight (Screen.CursorCol - (Screen.LeftCol+Screen.ScreenCols-1));
	}
}

void ProcessChar (BYTE InChar)
{
	RECT	rc;
	int		i,j;
	BOOL	OnScreen = ON_SCREEN(Screen.CursorRow, Screen.CursorCol);

	// You would think you only have to do this if we actually want to
	// change some of the text on the screen. In reality we do this
	// so that the cursor will be updated (on CR or LF).
	if (ON_SCREEN(Screen.CursorRow,Screen.CursorCol)) 
	{
		rc.top = ROWY(Screen.CursorRow);
		rc.bottom = ROWY(Screen.CursorRow+1);
		rc.left = COLX((Screen.CursorCol) ? (Screen.CursorCol-1) : 0);
		rc.right = COLX(Screen.CursorCol+1);
		InvalidateRect (hTermWnd, &rc, TRUE);
	}
		
	// Do something with the character.
	switch (InChar) 
	{
		case 0x08 :
			if (Screen.CursorCol) 
			{
				Screen.CursorCol--;
				CURPOS = TEXT(' ');
			}
			break;
		case 0x0d :
			Screen.CursorCol = 0;
			break;
		case 0x0a :
			Screen.CursorRow++;
			break;
		default :
			CURPOS = InChar;
			Screen.CursorCol++;
			break;
	}
	
	if (Screen.CursorCol == Screen.NumCols) 
	{
		Screen.CursorRow++;
		Screen.CursorCol = 0;
	}

	if (Screen.CursorRow == Screen.NumRows) 
	{
		// Shift data up.
		for (i=0; i < (Screen.NumRows-1); i++) 
		{
			memcpy (&(SCREENPOS(i,0)),&(SCREENPOS(i+1,0)),sizeof(TCHAR)*(Screen.NumCols));
		}		

		for (j=0;j<Screen.NumCols;j++)
		{
			SCREENPOS(i,j) = TEXT(' ');
		}

		Screen.CursorRow--;
				
		// For now just repaint.
		InvalidateRect (hTermWnd, NULL, TRUE);
	}

	// If we used to be visible we should keep it visible
	if (OnScreen && !ON_SCREEN(Screen.CursorRow,Screen.CursorCol)) 
	{
		PutCursorOnScreen ();
	}
}
